#include <math.h>
#include "mex.h"

#define PI 3.14159265358979
#define round(x) ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))

/* Copyright Fredrik Andersson, Lund University */

double ** newMat(int dim1, int dim2){
  int i, j;
  double  **matrice=(double **)malloc(dim1*sizeof(double *));
  for (i = 0; i < dim1; i++) matrice[i] = (double *)malloc(dim2*sizeof(double));
  for (i = 0; i < dim1; i++) for (j = 0; j < dim2; j++) matrice[i][j] = 0.0;
  return matrice;
}

void deleteMat(int dim1, double **matrice){
  int j; for (j = 0; j < dim1; j++)	free(matrice[j]);
  free(matrice);
}

float ** newMat_s(int dim1, int dim2){
  int i, j;
  float  **matrice=(float **)malloc(dim1*sizeof(float *));
  for (i = 0; i < dim1; i++){
    matrice[i] = (float *)malloc(dim2*sizeof(float));
  }
  for (i = 0; i < dim1; i++)
		for (j = 0; j < dim2; j++)
			matrice[i][j] = 0.0;
  return matrice;
}

void deleteMat_s(int dim1, float **matrice){
  int i;

  for (i = 0; i < dim1; i++)
		free(matrice[i]);
  free(matrice);
}


void Vproj(int N1, int N2, double lambda, int M_lambda, int M, double* x1, double* x2, double* Fr,double *Fi,double *Gr, double* Gi)
{
  int	j1,j2,k,l1,l2;
	double **Er, **Ei, **b;
  double g;
  double a,a1,a2;
  double s,s1,s2,t1,t2;
  int y1,y2;
  double xk1,xk2;
  double Grk, Gik; 
  Er=newMat(2*M_lambda+N1,2*M_lambda+N2);
  Ei=newMat(2*M_lambda+N1,2*M_lambda+N2);
  b=newMat(2*M_lambda+1,2*M_lambda+1);
	for(j1=-M_lambda;j1<=M_lambda;j1++)
		for(j2=-M_lambda;j2<=M_lambda;j2++)
			b[j1+M_lambda][j2+M_lambda]=exp(-lambda*(j1*j1+j2*j2));
	for(j1=M_lambda;j1<M_lambda+N1;j1++){
		for(j2=0;j2<M_lambda;j2++){
			Er[j1][j2]=Fr[j1-M_lambda+(j2-M_lambda+N2)*N1];
			Ei[j1][j2]=Fi[j1-M_lambda+(j2-M_lambda+N2)*N1];
		}
		for(j2=M_lambda;j2<M_lambda+N2;j2++){
			Er[j1][j2]=Fr[j1-M_lambda+(j2-M_lambda)*N1];
			Ei[j1][j2]=Fi[j1-M_lambda+(j2-M_lambda)*N1];
		}
		for(j2=M_lambda+N2;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Fr[j1-M_lambda+(j2-M_lambda-N2)*N1];
			Ei[j1][j2]=Fi[j1-M_lambda+(j2-M_lambda-N2)*N1];
		}
  }
 	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1+N1][j2];
			Ei[j1][j2]=Ei[j1+N1][j2];
		}
	}  
	for(j1=M_lambda+N1;j1<2*M_lambda+N1;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1-N1][j2];
			Ei[j1][j2]=Ei[j1-N1][j2];
		}
	}  
	for (k=0;k<M;k++){
    xk1=2*x1[k];
    xk2=2*x2[k];
    y1=(int)(xk1);
    y2=(int)(xk2);
    a1=exp(-lambda*( (xk1-y1)*(xk1-y1)));
    a2=exp(-lambda*( (xk2-y2)*(xk2-y2)));
    a=a1*a2;
    s1=exp(-lambda*( 2*(xk1-y1)*M_lambda));
    s2=exp(-lambda*( 2*(xk2-y2)*M_lambda));
    t1=exp(lambda*( 2*(xk1-y1)));
    t2=exp(lambda*( 2*(xk2-y2)));
    l1=y1+N1/2;
 		Grk=0.0;
		Gik=0.0;
  	for(j1=-M_lambda;j1<=M_lambda;j1++){
      s=s1*s2;
      l2=y2+N2/2;
			for(j2=-M_lambda;j2<=M_lambda;j2++){
        g=a*b[j1+M_lambda][j2+M_lambda]*s;
        Grk+=g*Er[l1][l2];
        Gik+=g*Ei[l1][l2];      
        l2++;
        s*=t2;        
			}
      s1*=t1;
      l1++;
		}
    Gr[k]=Grk*lambda/PI;
		Gi[k]=Gik*lambda/PI;
	}
  deleteMat(2*M_lambda+N1,Er);deleteMat(2*M_lambda+N1,Ei);deleteMat(2*M_lambda+1,b);
}


void Vproj_single_precision(int N1, int N2, double lambda, int M_lambda, int M, double* x1, double* x2, double* Fr,double *Fi,double *Gr, double* Gi)
{
  int	j1,j2,k,l1,l2;
	float **Er, **Ei, **b, *gr, *gi;
  float g;
  float a,a1,a2;
  float s,s1,s2,t1,t2;
  int y1,y2;
  float xk1,xk2;
  float Grk, Gik;
  float lambdas;lambdas=(float)(lambda);
  Er=newMat_s(2*M_lambda+N1,2*M_lambda+N2);
  Ei=newMat_s(2*M_lambda+N1,2*M_lambda+N2);
  b=newMat_s(2*M_lambda+1,2*M_lambda+1);  
  gr=(float*)malloc(M*sizeof (float));
  gi=(float*)malloc(M*sizeof (float));    
	for(j1=-M_lambda;j1<=M_lambda;j1++)
		for(j2=-M_lambda;j2<=M_lambda;j2++)
			b[j1+M_lambda][j2+M_lambda]=(float)(exp(-lambdas*(j1*j1+j2*j2)));
	for(j1=M_lambda;j1<M_lambda+N1;j1++){
		for(j2=0;j2<M_lambda;j2++){
			Er[j1][j2]=(float)Fr[j1-M_lambda+(j2-M_lambda+N2)*N1];
			Ei[j1][j2]=(float)Fi[j1-M_lambda+(j2-M_lambda+N2)*N1];
		}
		for(j2=M_lambda;j2<M_lambda+N2;j2++){
			Er[j1][j2]=(float)Fr[j1-M_lambda+(j2-M_lambda)*N1];
			Ei[j1][j2]=(float)Fi[j1-M_lambda+(j2-M_lambda)*N1];
		}
		for(j2=M_lambda+N2;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=(float)Fr[j1-M_lambda+(j2-M_lambda-N2)*N1];
			Ei[j1][j2]=(float)Fi[j1-M_lambda+(j2-M_lambda-N2)*N1];
		}
  }
 	for(j1=0;j1<M_lambda;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1+N1][j2];
			Ei[j1][j2]=Ei[j1+N1][j2];
		}
	}  
	for(j1=M_lambda+N1;j1<2*M_lambda+N1;j1++){
		for(j2=0;j2<2*M_lambda+N2;j2++){
			Er[j1][j2]=Er[j1-N1][j2];
			Ei[j1][j2]=Ei[j1-N1][j2];
		}
	}  
	for (k=0;k<M;k++){
    xk1=2*(float)x1[k];
    xk2=2*(float)x2[k];
    y1=(int)(xk1);
    y2=(int)(xk2);
    a1=(float)exp(-lambdas*( (xk1-y1)*(xk1-y1)));
    a2=(float)exp(-lambdas*( (xk2-y2)*(xk2-y2)));
    a=a1*a2;
    s1=(float)exp(-lambdas*( 2*(xk1-y1)*M_lambda));
    s2=(float)exp(-lambdas*( 2*(xk2-y2)*M_lambda));
    t1=(float)exp(lambdas*( 2*(xk1-y1)));
    t2=(float)exp(lambdas*( 2*(xk2-y2)));
    l1=y1+N1/2;
 		Grk=0.0;
		Gik=0.0;
  	for(j1=-M_lambda;j1<=M_lambda;j1++){
      s=s1*s2;
      l2=y2+N2/2;
			for(j2=-M_lambda;j2<=M_lambda;j2++){
        g=a*b[j1+M_lambda][j2+M_lambda]*s;
        Grk+=g*Er[l1][l2];
        Gik+=g*Ei[l1][l2];      
        l2++;
        s*=t2;        
			}
      s1*=t1;
      l1++;
		}
    gr[k]=Grk*lambdas/((float)PI);
		gi[k]=Gik*lambdas/((float)PI);
	}
  for(k=0;k<M;k++){
    Gr[k]=(double)(gr[k]);
    Gi[k]=(double)(gi[k]);
  }
  deleteMat_s(2*M_lambda+N1,Er);deleteMat_s(2*M_lambda+N1,Ei);deleteMat_s(2*M_lambda+1,b);free(gr);free(gi);
}


void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[] )     
{ 
  int N1,N2, M, M_lambda,j1,j2,M2;
  double lambda,g,epsilon;
  double *x1,*x2,*x1o,*x2o;
  double *Fr,*Fi;
  double *Gr,*Gi;
  double *br,*bi;
  mxArray* a[1];
  mxArray* b[1];
  mxArray* sor[2];
  mxArray* xs1A[1];
  mxArray* xs2A[1];
  mxArray* Vs[1];  
  double *xs1,*xs2;
  double *idx;
  double *Hr,*Hi;

  M=(int)(mxGetM(prhs[0])*mxGetN(prhs[0]));
  M2=(int)(mxGetM(prhs[1])*mxGetN(prhs[1]));
  if (nrhs <3) { 
   	mexErrMsgTxt("Input format: [xi1, xi2, data]"); 
  } else if (M!=M2) {
  	mexErrMsgTxt("Inconsistent size of positions and data"); 
  }     
  x1o=mxGetPr(prhs[0]);
  x2o=mxGetPr(prhs[1]);
  Fr = mxGetPr(prhs[2]);
  Fi = mxGetPi(prhs[2]);    
  x1=(double *)malloc(M*sizeof(double));
  x2=(double *)malloc(M*sizeof(double));  
  N1=(int)(mxGetN(prhs[2]));
  N2=(int)(mxGetM(prhs[2])); 
  if (nrhs<4)
    epsilon=1e-12;
  else
    epsilon=(mxGetPr(prhs[3]))[0]/10;
  xs1A[0]=mxCreateDoubleMatrix(M,1, mxREAL);xs1=mxGetPr(xs1A[0]);
  xs2A[0]=mxCreateDoubleMatrix(M,1, mxREAL);xs2=mxGetPr(xs2A[0]);
  for (j1=0;j1<M;j1++){
    xs1[j1]=x1o[j1]-(int)(x1o[j1]/(N1/2))*(N1/2);
    xs2[j1]=x2o[j1]-(int)(x2o[j1]/(N2/2))*(N2/2);
  }
  mexCallMATLAB(2,sor,1, xs1A, "sort");
  idx=mxGetPr(sor[1]);    
  for (j1=0;j1<M;j1++){
    x1[j1]=x1o[(int)(idx[j1])-1]-round((x1o[(int)(idx[j1])-1])/N1+1e-12)*N1;
    x2[j1]=x2o[(int)(idx[j1])-1]-round((x2o[(int)(idx[j1])-1])/N2+1e-12)*N2;    
  }  
  lambda=9.86960440108936/(-2*log(epsilon));
  M_lambda=(int)(sqrt(-log(epsilon)/lambda));  
  a[0] = mxCreateDoubleMatrix(2*N2,2*N1, mxCOMPLEX);
  b[0] = mxCreateDoubleMatrix(2*N2,2*N1, mxCOMPLEX);  
  Gr = mxGetPr(a[0]);
  Gi = mxGetPi(a[0]);
  for(j1=0;j1<2*N1;j1++) for(j2=0;j2<2*N2;j2++){Gr[j1+j2*N1]=0.0;Gi[j1+j2*N1]=0.0;}
	for(j1=0;j1<N1;j1++) for(j2=0;j2<N2;j2++){ 
    g=exp(9.86960440108936*((j1-N1/2)*(j1-N1/2)/(double)(N1*N1)+(j2-N2/2)*(j2-N2/2)/(double)(N2*N2))/(lambda*4));
    Gr[j2+N2/2+(j1+N1/2)*2*N2]=Fr[j2+j1*N2]*g;
    if (Fi!=NULL) Gi[j2+N2/2+(j1+N1/2)*2*N2]=Fi[j2+j1*N2]*g;
  }
  mexCallMATLAB(1,b,1, a, "fftshift");mexCallMATLAB(1,a,1, b, "fft2");mexCallMATLAB(1,b,1, a, "fftshift");  
  br = mxGetPr(b[0]);bi = mxGetPi(b[0]);    
  Vs[0] = mxCreateDoubleMatrix(M,1, mxCOMPLEX);
  Hr = mxGetPr(Vs[0]);
  Hi = mxGetPi(Vs[0]);
  if(epsilon<1e-8){
  	Vproj(2*N2,2*N1,lambda,M_lambda,M,x2,x1,br,bi,Hr,Hi);
  }
  else{
    Vproj_single_precision(2*N2,2*N1,lambda,M_lambda,M,x2,x1,br,bi,Hr,Hi);
  }
  plhs[0] = mxCreateDoubleMatrix(M,1, mxCOMPLEX);
  Gr = mxGetPr(plhs[0]);
  Gi = mxGetPi(plhs[0]);
  for (j1=0;j1<M;j1++){
    Gr[(int)(idx[j1])-1]=Hr[j1];
    Gi[(int)(idx[j1])-1]=Hi[j1];
  }  
  mxDestroyArray(a[0]);mxDestroyArray(b[0]);mxDestroyArray(sor[0]);mxDestroyArray(sor[1]);mxDestroyArray(Vs[0]);mxDestroyArray(xs1A[0]);mxDestroyArray(xs2A[0]);
  free(x1);free(x2);
  return;
}