%% demo_c2BayesEst.m
% 
% demo file for estimation of multifractality/intermeittency parameter c2
% for (2D) images and (1D) signals.
%
% Please refer to the help of the functions c2BayesImage.m and
% c2BayesSeries.m for details on usage and output.
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--- Sebastien Combrexelle, Toulouse, 2014-2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% If you use this code, please quote:
%
% @ARTICLE{CombrexelleTIP2015,
%   author = {Sebastien Combrexelle and Herwig Wendt and Nicolas Dobigeon and Jean-Yves Tourneret and Stephen McLaughlin and Patrice Abry},
%   title = {Bayesian Estimation of the Multifractality Parameter for Image Texture Using a Whittle Approximation},
%   journal = {IEEE Trans. Image Process},
%   volume = {},
%   number = {},
%   pages = {},
%   year = {2015},
%   note = {to appear}
% }
%
% @INPROCEEDINGS{CombrexelleEUSIPCO2015,
%   author = {Sebastien Combrexelle and Herwig Wendt and Jean-Yves Tourneret and Patrice Abry and Stephen McLaughlin},
%   title = {Bayesian estimation of the multifractality parameter for images via a closed-form Whittle likelihood},
%   booktitle = {European Signal Processing Conference (EUSIPCO)},
%   address = {Nice, France},
%   month = {September},
%   year = {2015}
% }
%
% @INPROCEEDINGS{WendtICASSP2013,
%   author = {Herwig Wendt and Nicolas Dobigeon and Jean-Yves Tourneret and Patrice Abry},
%   title = {Bayesian estimation for the multifractality parameter},
%   booktitle = {IEEE Int. Conf. Acoust., Speech, and Signal Proc. (ICASSP)},
%   address = {Vancouver, Canada},
%   month = {September},
%   year = {2013}
% }
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all; 
clc



pp=which('demo_c2BayesEst');if ispc;id=strfind(pp,'\');else;id=strfind(pp,'/');end;addpath(genpath(pp(1:id(end))));

% IMAGE
    % generate synthetic image
    N = 2^8; c2 = -0.08;
    data = mrw2D(0.8,sqrt(abs(c2)),N,1); 
    figure(1);clf;hold on;grid on;imagesc(data);colormap(bone); axis tight;title(['2D MRW c_2=',num2str(c2)]);disp(' ');
    disp(['estimation for image (N=',num2str(N),'^2, c2=',num2str(c2),'):']);
    
    % estimation
    Nwt = 2;    % number of vanishing moments (wavelet)
    j1=2; j2=4; % range of scales for estimation
    tic;
    [EST2,TAB2] = c2BayesImage(data,j1,j2,Nwt);  
    toc
    c2_est_image = EST2.c2

% TIME SERIES
    % generate synthetic time series
    N = 2^12; c2 = -0.08;H=0.72;
    [data0] = mrw(H,sqrt(abs(c2)),N+1,N+1,2) ;  data = cumsum(data0(2:end));
    figure(2);clf;hold on;grid on;plot(data);axis tight;title(['1D MRW c_2=',num2str(c2)]);disp(' ');
    disp(['estimation for time series (N=',num2str(N),', c2=',num2str(c2),'):']);
    
    % estimation
    Nwt = 2;    % number of vanishing moments (wavelet)
    j1=2; j2=7; % range of scales for estimation
    tic;
    [EST1,TAB1] = c2BayesSeries(data,j1,j2,Nwt); 
    toc
    c2_est_series = EST1.c2
    
    
